/**
  *  IMPORTANT NOTE!
  *        RAM,   , 
  *     VECT_TAB_SRAM,  
  *  .     ,
  *    Flash-.
  *    <system_*.c>,  SystemInit() 
  *   SCB->VTOR.
  *       RAM  , ,
  *  SCB->VTOR = 0x20000000;
  */

/**
 *       .
 *     USER button   /
 *   BLUE LED.
 *
 *     STM32VLDISCOVERY
 *   :
 *
 *    PA0,    . 1
 *   ,     
 *    RC-;
 *
 *     PC8   
 * . 1   .
 */

// PA0 - USER button;
// PC8 - blue led;
// PC9 - green led.

#include "stm32f10x.h"

//    PC8  ( 
//     ).
void toggle_blue_led()
{
	GPIOC->ODR^=1<<8;
}

//      EXTI0.
extern "C" void EXTI0_IRQHandler()
{
	//     EXTI->PR    1.
	EXTI->PR=1;

	//   ,   ,  .
	toggle_blue_led();
}



int main(void)
{
	//  NVIC -    :
	//       
	//  .
	NVIC_SetPriority(EXTI0_IRQn, 0); //   .
	NVIC_EnableIRQ(EXTI0_IRQn);

	//      :
	// AFIO (    AFIO->EXTICR[]),
	// GPIOA, GPIOC.
	// EXTI     .
	RCC->APB2ENR|=
		RCC_APB2ENR_AFIOEN|
		RCC_APB2ENR_IOPAEN|
		RCC_APB2ENR_IOPCEN;

	//    -.
	// 1. PA0: float in; (CNF[1:0], MODE[1:0])=0100 (bin)=0x4
	GPIOA->CRL=GPIOA->CRL&~0xF|0x4;
	// 2. PC8: push-pull out, 2MHz max; (CNF, MODE)=0010 (bin)=0x2
	GPIOC->CRH=GPIOC->CRH&~0xF|0x2;

	//     PY[x] 
	//  x    EXTI;
	//    Y=A ( 0), x=0, ..  
	//   AFIO->EXTICR[y]  
	//   :
	// AFIO->EXTICR[0]=AFIO->EXTICR[0]&~AFIO_EXTICR1_EXTI0|AFIO_EXTICR1_EXTI0_PA;

	//      EXTI0,   - .
	EXTI->IMR=1;
	// EXTI->IMR|=1; //      .

	//   0    .
	EXTI->RTSR=1;
	// EXTI->RTSR|=1; //      .

	//      USER button 
	//   /  BLUE LED.

	int i = 0;
	/* Infinite loop */
	while (1)
	{
		i++;
	}
}
